@@ -12,7 +12,7 @@ ci.putenv("OUNIT_OUTPUT_HTML_DIR", dist.make_filename("ounit-log-$(suite_name).h
 ci.putenv("OUNIT_OUTPUT_JUNIT_FILE", dist.make_filename("junit-$(suite_name).xml"))
 ci.putenv("OUNIT_OUTPUT_FILE", dist.make_filename("ounit-log-$(suite_name)-$(shard_id).txt"))
 
-oasis.std_process("--enable-tests")
+oasis.std_process("--enable-tests", "--enable-devel")
 
 -- Create documentation package.
 ci.exec("make", "doc-dist")
