@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -21,16 +21,12 @@
 (******************************************************************************)
 
 
-TYPE_CONV_PATH "OASISExpr"
-
-
 open OASISGettext
+open OASISUtils
 
 
-type test = string with odn
-
-
-type flag = string with odn
+type test = string
+type flag = string
 
 
 type t =
@@ -40,10 +36,9 @@ type t =
   | EOr of t * t
   | EFlag of flag
   | ETest of test * string
-  with odn
 
 
-type 'a choices = (t * 'a) list with odn
+type 'a choices = (t * 'a) list
 
 
 let eval var_get t =
@@ -117,9 +112,6 @@ let choose ?printer ?name var_get lst =
 (* END EXPORT *)
 
 
-open OASISUtils
-
-
 let tests =
   [
     "os_type";
@@ -229,10 +221,11 @@ let if_then_else t choices_if choices_else =
 
 
 let rec to_string =
+  let id_or_qstring s = if is_varname s then s else Printf.sprintf "%S" s in
   function
     | EBool b -> string_of_bool b
-    | EFlag nm -> "flag("^nm^")"
-    | ETest (nm, vl) -> nm^"("^vl^")"
+    | EFlag nm -> "flag("^(id_or_qstring nm)^")"
+    | ETest (nm, vl) -> nm^"("^(id_or_qstring vl)^")"
 
     | EOr (e1, e2) ->
         (to_string e1)^" || "^(to_string e2)
@@ -255,3 +248,19 @@ let rec to_string =
 let string_of_choices f lst =
   "["^(String.concat "; "
          (List.rev_map (fun (e, vl) -> (to_string e)^" -> "^(f vl)) lst))^"]"
+
+
+let odn_of_choices odn_of_a x =
+  let open OASISDataNotation in
+  let rec odn_of_t =
+    function
+    | EBool v0 -> VRT ("OASISExpr.EBool", [ of_bool v0 ])
+    | ENot v0 -> VRT ("OASISExpr.ENot", [ odn_of_t v0 ])
+    | EAnd ((v1, v0)) ->
+        VRT ("OASISExpr.EAnd", [ odn_of_t v1; odn_of_t v0 ])
+    | EOr ((v1, v0)) -> VRT ("OASISExpr.EOr", [ odn_of_t v1; odn_of_t v0 ])
+    | EFlag v0 -> VRT ("OASISExpr.EFlag", [ STR v0 ])
+    | ETest ((v1, v0)) ->
+        VRT ("OASISExpr.ETest", [ of_string (string_of_test v1); of_string v0 ])
+  in
+  of_list (fun (v1, v0) -> TPL [ odn_of_t v1; odn_of_a v0 ]) x
