@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -22,6 +22,7 @@
 
 
 open Filename
+open OASISGettext
 
 
 module Unix = OASISUnixPath
@@ -30,15 +31,15 @@ module Unix = OASISUnixPath
 let make =
   function
     | [] ->
-        invalid_arg "OASISHostPath.make"
+      invalid_arg "OASISHostPath.make"
     | hd :: tl ->
-        List.fold_left Filename.concat hd tl
+      List.fold_left Filename.concat hd tl
 
 
 let of_unix ufn =
-  if Sys.os_type = "Unix" then
-    ufn
-  else
+  match Sys.os_type with
+  | "Unix" | "Cygwin" -> ufn
+  | "Win32" ->
     make
       (List.map
          (fun p ->
@@ -49,69 +50,71 @@ let of_unix ufn =
             else
               p)
          (OASISString.nsplit ufn '/'))
+  | os_type ->
+    OASISUtils.failwithf
+      (f_ "Don't know the path format of os_type %S when translating unix \
+           filename. %S")
+      os_type ufn
 
 
 (* END EXPORT *)
 
 
 let fn_norm fn =
-  match OASISPath_intern.fn_reduce []
-          (OASISPath_intern.fn_reader fn) with
-    | (`RootRelative str) :: tl ->
-        (`Root str) :: `CurrentDir :: tl
-    | lst ->
-        lst
+  match OASISPath_intern.fn_reduce [] (OASISPath_intern.fn_reader fn) with
+  | (`RootRelative str) :: tl -> (`Root str) :: `CurrentDir :: tl
+  | lst -> lst
+
 
 let compare fn1 fn2 =
   let fn_string =
     function
-      | `Root str
-      | `RootRelative str
-      | `Component str ->
-          str
-      | `CurrentDir ->
-          "."
-      | `ParentDir ->
-          ".."
+    | `Root str | `RootRelative str | `Component str -> str
+    | `CurrentDir -> "."
+    | `ParentDir -> ".."
   in
   let rec compare' =
     function
-      | (hd1 :: tl1), (hd2 :: tl2) ->
-          if hd1 = hd2 then
-            compare' (tl1, tl2)
-          else
-            String.compare (fn_string hd1) (fn_string hd2)
-      | [], [] ->
-          0
-      | _ :: _, [] ->
-          1
-      | [], _ :: _ ->
-          -1
+    | (hd1 :: tl1), (hd2 :: tl2) ->
+      if hd1 = hd2 then
+        compare' (tl1, tl2)
+      else
+        String.compare (fn_string hd1) (fn_string hd2)
+    | [], [] -> 0
+    | _ :: _, [] -> 1
+    | [], _ :: _ -> -1
   in
-    compare' (fn_norm fn1, fn_norm fn2)
+  compare' (fn_norm fn1, fn_norm fn2)
 
 
 let to_unix hfn =
-  if Sys.os_type = "Unix" then
-    hfn
-  else
-  let rec to_unix_aux =
-    function
+  match Sys.os_type with
+  | "Unix" | "Cygwin" -> hfn
+  | "Win32" ->
+    let rec to_unix_aux =
+      function
       | `Root str :: _
       | `RootRelative str :: _ ->
-          OASISUtils.failwithf
-            "Cannot translate %S to unix filename, it contains a root \
-             reference (%S)." hfn str
+        OASISUtils.failwithf
+          (f_ "Cannot translate %S to unix filename, it contains a root \
+               reference (%S).")
+          hfn
+          str
       | `Component str :: tl ->
-          str :: (to_unix_aux tl)
+        str :: (to_unix_aux tl)
       | `CurrentDir :: tl ->
-          "." :: (to_unix_aux tl)
+        "." :: (to_unix_aux tl)
       | `ParentDir :: tl ->
-          ".." :: (to_unix_aux tl)
+        ".." :: (to_unix_aux tl)
       | [] ->
-          []
-  in
+        []
+    in
     OASISUnixPath.make (to_unix_aux (fn_norm hfn))
+  | os_type ->
+    OASISUtils.failwithf
+      (f_ "Don't know the path format of os_type %S when translating host \
+           filename. %S")
+      os_type hfn
 
 
 let add_extension fn ext =
@@ -119,8 +122,8 @@ let add_extension fn ext =
 
 
 module Map = OASISUtils.MapExt.Make (
-struct
-       type t = Unix.host_filename
+  struct
+    type t = Unix.host_filename
 
-       let compare = compare
-end)
+    let compare = compare
+  end)
