@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -23,43 +23,46 @@
 
 (** Manipulate sections.
     @author Sylvain Le Gall
-  *)
+*)
 
 
 open OASISTypes
 
+(** Convert [section_kind] to string. *)
+val string_of_section_kind: section_kind -> string
+
 
 (** {2 Manipulate a section} *)
 
 
 (** Extract generic information
-  *)
+*)
 val section_kind_common:  section -> section_kind * common_section
 
 
 (** Common section of a section
-  *)
+*)
 val section_common: section -> common_section
 
 
 (** Set the common part of a section
-  *)
+*)
 val section_common_set: common_section -> section -> section
 
 
 (** Key used to identify section
-  *)
+*)
 val section_id: section -> section_kind * name
 
 
 (** Convert a section to a short string (only informations returned by
     {!section_id}
-  *)
+*)
 val string_of_section: section -> string
 
 
 (** Find a section
-  *)
+*)
 val section_find: section_kind * name -> section list -> section
 
 
@@ -67,8 +70,8 @@ val section_find: section_kind * name -> section list -> section
 
 
 (** Comparable section, we only rely on section_id
-   for comparison. {b Not exported}
-  *)
+    for comparison. {b Not exported}
+*)
 module CSection:
 sig
   type t = section
