@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -26,39 +26,14 @@
   *)
 
 
-TYPE_CONV_PATH "MyOCamlbuildBase"
-
-
 open Ocamlbuild_plugin
 module OC = Ocamlbuild_pack.Ocaml_compiler
 
 
-type dir = string with odn
-type file = string with odn
-type name = string with odn
-type tag = string with odn
-
-
-(* END EXPORT *)
-let rec odn_of_spec =
-  let vrt nm lst =
-    ODN.VRT ("Ocamlbuild_plugin."^nm, lst)
-  in
-  let vrt_str nm str =
-    vrt nm [ODN.STR str]
-  in
-    function
-      | N     -> vrt "N" []
-      | S lst -> vrt "S" [ODN.of_list odn_of_spec lst]
-      | A s   -> vrt_str "A" s
-      | P s   -> vrt_str "P" s
-      | Px s  -> vrt_str "Px" s
-      | Sh s  -> vrt_str "Sh" s
-      | V s   -> vrt_str "V" s
-      | Quote spc -> vrt "Quote" [odn_of_spec spc]
-      | T _ ->
-          assert false
-(* START EXPORT *)
+type dir = string
+type file = string
+type name = string
+type tag = string
 
 
 type t =
@@ -70,12 +45,71 @@ type t =
        * directory.
        *)
       includes:  (dir * dir list) list;
-    } with odn
+    }
 
 
-let env_filename =
-  Pathname.basename
-    BaseEnvLight.default_filename
+(* END EXPORT *)
+let odn_of_t v =
+  let open OASISDataNotation in
+  let rec odn_of_spec =
+    let vrt nm lst =
+      VRT ("Ocamlbuild_plugin."^nm, lst)
+    in
+    let vrt_str nm str =
+      vrt nm [STR str]
+    in
+      function
+        | N     -> vrt "N" []
+        | S lst -> vrt "S" [of_list odn_of_spec lst]
+        | A s   -> vrt_str "A" s
+        | P s   -> vrt_str "P" s
+        | Px s  -> vrt_str "Px" s
+        | Sh s  -> vrt_str "Sh" s
+        | V s   -> vrt_str "V" s
+        | Quote spc -> vrt "Quote" [odn_of_spec spc]
+        | T _ ->
+            assert false
+  in
+  REC ("MyOCamlbuildBase",
+    [ ("lib_ocaml",
+       ((fun x ->
+           of_list
+             (fun (v2, v1, v0) ->
+                TPL
+                  [ of_string v2; of_list of_string v1;
+                    of_list of_string v0 ])
+             x)
+          v.lib_ocaml));
+      ("lib_c",
+       ((fun x ->
+           of_list
+             (fun (v2, v1, v0) ->
+                TPL
+                  [ of_string v2; of_string v1;
+                    of_list of_string v0 ])
+             x)
+          v.lib_c));
+      ("flags",
+       ((fun x ->
+           of_list
+             (fun (v1, v0) ->
+                TPL
+                  [ of_list of_string v1;
+                    OASISExpr.odn_of_choices odn_of_spec v0 ])
+             x)
+          v.flags));
+      ("includes",
+       ((fun x ->
+           of_list
+             (fun (v1, v0) ->
+                TPL
+                  [ of_string v1; of_list of_string v0 ])
+             x)
+          v.includes)) ])
+(* START EXPORT *)
+
+
+let env_filename = Pathname.basename BaseEnvLight.default_filename
 
 
 let dispatch_combine lst =
@@ -94,12 +128,7 @@ let nm_libstubs nm =
 
 
 let dispatch t e =
-  let env =
-    BaseEnvLight.load
-      ~filename:env_filename
-      ~allow_empty:true
-      ()
-  in
+  let env = BaseEnvLight.load ~allow_empty:true () in
     match e with
       | Before_options ->
           let no_trailing_dot s =
@@ -163,18 +192,19 @@ let dispatch t e =
                  flag ["link"; "library"; "ocaml"; "native"; tag_libstubs lib]
                    (S[A"-cclib"; A("-l"^(nm_libstubs lib))]);
 
-                 flag ["link"; "program"; "ocaml"; "byte"; tag_libstubs lib]
-                   (S[A"-dllib"; A("dll"^(nm_libstubs lib))]);
+                 if bool_of_string (BaseEnvLight.var_get "native_dynlink" env) then
+                   flag ["link"; "program"; "ocaml"; "byte"; tag_libstubs lib]
+                       (S[A"-dllib"; A("dll"^(nm_libstubs lib))]);
 
                  (* When ocaml link something that use the C library, then one
                     need that file to be up to date.
                     This holds both for programs and for libraries.
                   *)
-		 dep ["link"; "ocaml"; tag_libstubs lib]
-		     [dir/"lib"^(nm_libstubs lib)^"."^(!Options.ext_lib)];
+                 dep ["link"; "ocaml"; tag_libstubs lib]
+                   [dir/"lib"^(nm_libstubs lib)^"."^(!Options.ext_lib)];
 
-		 dep  ["compile"; "ocaml"; tag_libstubs lib]
-		      [dir/"lib"^(nm_libstubs lib)^"."^(!Options.ext_lib)];
+                 dep  ["compile"; "ocaml"; tag_libstubs lib]
+                   [dir/"lib"^(nm_libstubs lib)^"."^(!Options.ext_lib)];
 
                  (* TODO: be more specific about what depends on headers *)
                  (* Depends on .h files *)
