@@ -1,7 +1,7 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2011-2016, Sylvain Le Gall                                   *)
 (* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
@@ -33,17 +33,12 @@ open OASISString
 open BaseEnv
 open OCamlbuildCommon
 open BaseStandardVar
-open BaseMessage
 
 
-TYPE_CONV_PATH "OCamlbuildPlugin"
+let cond_targets_hook = ref (fun lst -> lst)
 
 
-let cond_targets_hook =
-  ref (fun lst -> lst)
-
-
-let build extra_args pkg argv =
+let build ~ctxt args pkg argv =
   (* Return the filename in build directory *)
   let in_build_dir fn =
     Filename.concat
@@ -107,8 +102,8 @@ let build extra_args pkg argv =
                         (List.map
                            (List.filter
                               (fun fn ->
-                               ends_with ".cmo" fn
-                               || ends_with ".cmx" fn))
+                               ends_with ~what:".cmo" fn
+                               || ends_with ~what:".cmx" fn))
                            unix_files))
                  in
 
@@ -123,10 +118,8 @@ let build extra_args pkg argv =
 
            | Executable (cs, bs, exec) when var_choose bs.bs_build ->
                begin
-                 let evs, unix_exec_is, unix_dll_opt =
-                   BaseBuilt.of_executable
-                     in_build_dir_of_unix
-                     (cs, bs, exec)
+                 let evs, _, _ =
+                   BaseBuilt.of_executable in_build_dir_of_unix (cs, bs, exec)
                  in
 
                  let target ext =
@@ -140,7 +133,7 @@ let build extra_args pkg argv =
                      (* Fix evs, we want to use the unix_tgt, without copying *)
                      List.map
                        (function
-                          | BaseBuilt.BExec, nm, lst when nm = cs.cs_name ->
+                          | BaseBuilt.BExec, nm, _ when nm = cs.cs_name ->
                               BaseBuilt.BExec, nm,
                               [[in_build_dir_of_unix unix_tgt]]
                           | ev ->
@@ -184,27 +177,30 @@ let build extra_args pkg argv =
                 (List.length fns))
              (String.concat (s_ " or ") (List.map (Printf.sprintf "'%s'") fns)))
       lst;
-      (BaseBuilt.register bt bnm lst)
+      (BaseBuilt.register ~ctxt bt bnm lst)
   in
 
   (* Run the hook *)
   let cond_targets = !cond_targets_hook cond_targets in
 
   (* Run a list of target... *)
-  run_ocamlbuild (List.flatten (List.map snd cond_targets) @ extra_args) argv;
+  run_ocamlbuild
+    ~ctxt
+    {args with extra = List.flatten (List.map snd cond_targets) @ args.extra}
+    argv;
   (* ... and register events *)
   List.iter check_and_register (List.flatten (List.map fst cond_targets))
 
 
-let clean pkg extra_args  =
-  run_clean extra_args;
+let clean ~ctxt pkg args  =
+  run_clean ~ctxt args;
   List.iter
     (function
        | Library (cs, _, _) ->
-           BaseBuilt.unregister BaseBuilt.BLib cs.cs_name
+           BaseBuilt.unregister ~ctxt BaseBuilt.BLib cs.cs_name
        | Executable (cs, _, _) ->
-           BaseBuilt.unregister BaseBuilt.BExec cs.cs_name;
-           BaseBuilt.unregister BaseBuilt.BExecLib cs.cs_name
+           BaseBuilt.unregister ~ctxt BaseBuilt.BExec cs.cs_name;
+           BaseBuilt.unregister ~ctxt BaseBuilt.BExecLib cs.cs_name
        | _ ->
            ())
     pkg.sections
@@ -217,10 +213,8 @@ open OASISFileTemplate
 open OASISUtils
 open OASISMessage
 open OASISGettext
-open ODN
 open OASISPlugin
 open OASISTypes
-open OASISSchema
 open MyOCamlbuildBase
 open Ocamlbuild_plugin
 open OCamlbuildId
@@ -965,7 +959,7 @@ let add_ocamlbuild_files ctxt pkg =
 
                  let ctxt =
                    match obj.obj_modules with
-                     | [ m ] -> ctxt
+                     | [ _ ] -> ctxt
                      | _ -> (* generate mlpack file *)
                          let fn_base = prepend_bs_path bs cs.cs_name in
                          let fn =
@@ -1119,7 +1113,7 @@ let add_ocamlbuild_files ctxt pkg =
             (
               Format.fprintf Format.str_formatter
                 "@[<hv2>let package_default =@ %a@,@];;"
-                (pp_odn ~opened_modules:["Ocamlbuild_plugin"])
+                (OASISDataNotation.pp_odn ~opened_modules:["Ocamlbuild_plugin"])
                 (MyOCamlbuildBase.odn_of_t myocamlbuild_t);
               Format.flush_str_formatter ()
             );
@@ -1143,18 +1137,18 @@ let generator =
   ocamlbuild_common_generator pivot_data OASISPackage.schema all_id
 
 let doit ctxt pkg =
-  let extra_args =
-    extra_args_ocamlbuild_common ~ctxt:ctxt.ctxt pkg
+  let args, ctxt =
+    args_ocamlbuild_common ctxt pkg
       (generator pkg.schema_data)
   in
   let ctxt = add_ocamlbuild_files ctxt pkg in
     ctxt,
     {
       chng_moduls       = [OCamlbuildData.ocamlbuildsys_ml];
-      chng_main         = ODNFunc.func_with_arg build
+      chng_main         = OASISDataNotation.func_with_arg_ctxt build
                             "OCamlbuildPlugin.build"
-                            extra_args odn_of_extra_args;
-      chng_clean        = Some (ODNFunc.func clean "OCamlbuildPlugin.clean");
+                            args (OCamlbuildCommon.odn_of_args);
+      chng_clean        = Some (OASISDataNotation.func clean "OCamlbuildPlugin.clean");
       chng_distclean    = None;
     }
 
